# coding=utf-8
"""
Crée des colonnes de forme anulaires

USAGE

    python -m thyrsis.create_rings database.sqlite center file
    
    avec :
        center = 'nom du point' (supposé dans la base) ou coordonnées x,y
        file = {rayon, début, duree, flux} pour chaque anneau
    
"""
from builtins import str
from builtins import range

import math
from ..log import logger

# je préfère créer une fonction que le script appelle, plutôt
# que faire un scrip monolythique
#
# ça permet d'utiliser la fonction dans l'interface si besoin
#
# ici j'ai décidé de mettre un curseur comme point d'entrée
# ça signifie que je ne peux pas commiter mes modification (c'est l'object connection
# qui le permet, tu peux donc annuller tes modifications
#
# c'est le script (if __name__ == "__main__": ci-dessous qui crée la connection
# et le curseur et qui commit les modifications faites par la fonction

def create_rings(cur, center, debut, fileName):
    """Create ring from a center coordinates

    :param cur: cursor on a sqlite connection
    :type cur: sqlite3.Cursor
    :param center: center point
    :type center: tuple
    :param debut: begin date
    :type debut: string
    :param fileName: file path ([radius, begin date, duration, flux] per line)
    :type fileName: string
    """
    project_SRID = str(cur.execute("SELECT srid FROM geometry_columns WHERE f_table_name = 'noeuds'").fetchone()[0])
    an2s = 365.25*86400.0
    rav = 0.
    num = 1
    with open(fileName) as fil:
        next(fil)
        for line in fil:
            spl = line.split()
            r = float(spl[0])
            duree = str(float(spl[2])/an2s)+' year'
            flux = float(spl[3])
            logger.debug(r, duree, flux)
            cur.execute("""insert into injections(geometry, debut, duree, nom, input) 
                values(CastToMultiPolygon(GeomFromText('%s', %s)), '%s', '%s', '%s', %e)"""%(wkt_ring(center, rav, r),
                project_SRID, datetime.strptime(debut, "%d/%m/%Y"), duree, 'A'+str(num), flux))
            rav = r
            num += 1


def wkt_ring(center, r, R):
    """return wkt ring (polygon with hole) beween r1 and r2
    r < R
    
    :param center: center point
    :type center: tuple
    :param r: interior radius
    :type r: float
    :param R: exterior radius
    :type R: float
    """
    
    nb_seg = 32
    nb_pt = nb_seg + 1 # il faut fermer le contour
    d_theta = 2*math.pi/32
    x, y = center

    exterior = ["%f %f"%(x+R*math.cos(t*d_theta), y+R*math.sin(t*d_theta)) for t in range(nb_pt)]
    # note: le trou doit tourner en sens inverse
    interior = ["%f %f"%(x+r*math.cos(t*d_theta), y+r*math.sin(t*d_theta)) for t in reversed(list(range(nb_pt)))]

    return "POLYGON((%s),(%s))"%(','.join(exterior), ','.join(interior))

if __name__ == "__main__":

    import sys
    from ..database import sqlite
    from datetime import datetime

    if len(sys.argv) == 2 and sys.argv[1] in ["-h","--help"]:
        help(sys.modules[__name__])
        exit(0)

    if len(sys.argv) < 3:
        error("wrong number of arguments (try create_rings.py --help)")
        exit(1)

    conn = sqlite.connect(sys.argv[1])
    cur = conn.cursor()

    debut = '01/01/1958'
    duree = '100 year'
    cur.execute("""UPDATE simulations SET debut = '%s',
            type_injection = 'flux'"""%(datetime.strptime(debut, "%d/%m/%Y")))
    cur.execute("UPDATE dates_resultats SET duree = '%s'"%(duree))

    nom_point = sys.argv[2]
    fichier = sys.argv[3]

    if ',' in nom_point:
        spl = nom_point.split(',')
        center = float(spl[0]), float(spl[1])
    else:
        center = cur.execute("select x(geometry), y(geometry) from points_interet where nom='%s'"%(nom_point)).fetchone()
        if not center:
            raise RuntimeError("Point %s non trouvé"%(nom_point))

    logger.debug("center = ", center)
    create_rings(cur, center, debut, sys.argv[3])
    conn.commit()
    
